using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Threading;
using System.Diagnostics;

namespace Utils.Logs
{
    /// <summary>
    /// Logs writing class
    /// </summary>   
    /// <remarks>This class is thread safe</remarks>
    public class Log
    {
        #region Variables
        /// <summary>
        /// Logs file
        /// </summary>
        private FileInfo vFileToLog;
        /// <summary>
        /// Reference on the unique instance of Logs
        /// </summary>
        private static readonly Log refLog = new Log();
        /// <summary>
        /// StopWatch to calculate elapsed time
        /// </summary>
        private Stopwatch vStopWatch = new Stopwatch();
        /// <summary>
        /// If TRUE and Compilation is on DEBUG it'll log otherwise not.
        /// If FALSE it'll log anyways.
        /// </summary>
        private bool vDebugMode = true;
        #endregion

        #region Constructeurs
        /// <summary>
        /// Default constructor
        /// </summary>
        /// <exception cref="Exception">Une erreur est survenue et il est ompossible de traiter les logs</exception>
        protected Log() { }
        /// <summary>
        /// Gives the reference of the object Logs.
        /// </summary>
        /// <returns>A reference on the Logs object</returns>
        public static Log GetInstance()
        {
            refLog.TestFile();
            return refLog;
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Write a log without title ant the type if "Misc"
        /// </summary>
        /// <param name="_Msg">Logs Message</param>
        public void Write(string _Msg)
        {
            try
            {
                Write(_Msg, "", LogType.Miscellaneous);
            }
            catch (Exception) { throw; }
        }
        /// <summary>
        /// Write a log of "Misc" type
        /// </summary>
        /// <param name="_Title">Logs title</param>
        /// <param name="_Msg">Logs message</param>
        public void Write(string _Msg, string _Title)
        {
            try
            {
                Write(_Msg, _Title, LogType.Miscellaneous);
            }
            catch (Exception) { throw; }
        }
        /// <summary>
        /// Write a log and automatically find informations about the method within the log was asked
        /// </summary>
        /// <param name="_Msg">Message to log in the file</param>
        /// <param name="_Sender">The object within the log was asked</param>
        /// <param name="_Type">Logs type</param>
        public void Write(string _Msg, object _Sender, LogType _Type)
        {
            StackFrame sf = new StackFrame(2, false);
            string Title = "[" + _Sender.GetType().ToString() + "] " + sf.GetMethod();
            this.Write(_Msg, Title, _Type);
        }
        /// <summary>
        /// Write a log and automatically find informations about the method within the log was asked
        /// </summary>
        ///<param name="_Ex">The exception to log</param>   
        /// <param name="_Sender">The object within the log was asked</param>
        /// <param name="_Type">Logs type</param>
        public void Write(Exception _Ex, object _Sender, LogType _Type)
        {
            Write(_Ex.Message, _Sender, _Type);
        }
        /// <summary>
        /// Write a log and automatically find informations about the method within the log was asked
        /// </summary>
        /// <param name="_Msg">Message to log in the file</param>        
        /// <param name="_Type">Logs type</param>
        /// <param name="_ClassType">Type of the class which asked the log</param>        
        public void Write(string _Msg, Type _ClassType, LogType _Type)
        {
            StackFrame sf = new StackFrame(2, false);
            string Title = "[" + _ClassType.ToString() + "] " + sf.GetMethod();
            this.Write(_Msg, Title, _Type);
        }
        /// <summary>
        /// Write a log and automatically find informations about the method within the log was asked
        /// </summary>
        ///<param name="_Ex">The exception to log</param>     
        /// <param name="_Type">Logs type</param>
        /// <param name="_ClassType">Type of the class which asked the log</param>        
        public void Write(Exception _Ex, Type _ClassType, LogType _Type)
        {
            Write(_Ex.Message, _ClassType, _Type);
        }
        /// <summary>
        /// Write a log
        /// </summary>
        /// <param name="_Type">Logs type</param>
        /// <param name="_Title">Logs title</param>
        /// <param name="_Msg">Logs message</param>
        /// <exception cref="Exception">If a log writing error occures</exception>
        public void Write(string _Msg, string _Title, LogType _Type)
        {
            if (DebugMode)
            {
#if  DEBUG
#else 
                    return;
#endif
            }
            lock (this)
            {
                #region Type
                string Type = _Type.ToString();
                int size = LogType.Miscellaneous.ToString().Length - Type.Length;
                string wSpace = "";
                for (int i = 0; i < size; i++)
                    wSpace += " ";
                Type += wSpace;
                #endregion

                #region Title
                wSpace = "";
                size = 50 - _Title.Length;
                for (int i = 0; i < size; i++)
                    wSpace += " ";
                _Title += wSpace;
                #endregion

                #region Enlevers les Sparateurs
                _Msg = _Msg.Replace(CellSeparator, " ");
                _Msg = _Msg.Replace(LineSeparator, " ");
                #endregion

                #region Entre  logger
                string ToWrite = DateTime.Now.ToString() + CellSeparator
                    + Type + CellSeparator + _Title + CellSeparator + _Msg + CellSeparator;

                try
                {
                    StreamWriter sw = vFileToLog.AppendText();
                    sw.WriteLine(ToWrite);
                    sw.Close();
                }
                catch (ThreadStateException ex) { Log.Instance.Write(ex.Message, this, LogType.Error); }
                catch (ThreadAbortException) { }
                catch (Exception ex)
                {
                    throw new Exception("Erreur dans l'criture des logs" + ex.Message);
                }

                #endregion
            }
        }
        /// <summary>
        /// Write a log
        /// </summary>
        /// <param name="_Type">Logs type</param>
        /// <param name="_Title">Logs title</param>
        ///<param name="_Ex">The exception to log</param>     
        /// <exception cref="Exception">If a log writing error occures</exception>
        public void Write(Exception _Ex, string _Title, LogType _Type)
        {
            Write(_Ex.Message, _Title, _Type);
        }
        /// <summary>
        /// The elapsed time is restet to 0 bafore be started
        /// </summary>
        public void RestartWatch()
        {
            vStopWatch.Reset();
            vStopWatch.Start();
        }
        /// <summary>
        /// Starts a StopWach to measure time
        /// </summary>
        /// <returns>TRUE if the StopWatch started otherwize FALSE</returns>
        public bool StartWatch()
        {
            if (vStopWatch.IsRunning) return false;
            vStopWatch.Start();
            return true;
        }
        /// <summary>
        /// Stops the started StopWatch and log the eslapsed time into the log file
        /// </summary>
        /// <param name="_Title">Title of the log</param>
        /// <returns>TRUE if the StopWatch stoped and it loged otherwize FALSE</returns>
        public bool StopWatch(string _Title)
        {
            if (!vStopWatch.IsRunning) return false;
            vStopWatch.Stop();
            Write("Elapsed time: " + vStopWatch.Elapsed.Minutes + " min " + vStopWatch.Elapsed.Seconds
                + " sec " + vStopWatch.Elapsed.Milliseconds + " msec", _Title, LogType.Debug);
            return true;
        }
        /// <summary>
        /// Reset the StopWatch
        /// </summary>        
        public void ResetWatch()
        {
            vStopWatch.Reset();
        }
        /// <summary>
        /// Check the size, the file name and the dorectory of the the log file and create a new file or directory 
        /// if necessary
        /// </summary>
        private void TestFile()
        {

            try
            {
                //Si le dosier n'existe pas
                if (!Directory.Exists(CommandPath + @"\Log"))
                {
                    string path = CommandPath + @"\Log";
                    Console.WriteLine(path);
                    Directory.CreateDirectory(path);
                }

                //Si le ficher est plus grand que 250k crer un nouveau
                if (Directory.GetFiles(CommandPath + @"\Log\").Length != 0)
                {
                    string[] NameOfFiles = Directory.GetFiles(CommandPath + @"\Log\");

                    FileInfo fi = new FileInfo(NameOfFiles[NameOfFiles.Length - 1]);

                    if (fi.Length > 250000)
                    {
                        string FilePath = CommandPath + @"\Log\Log_" + NameOfFiles.Length + ".wlg";
                        FileInfo newF = new FileInfo(FilePath);
                        newF.Create().Close();
                        StreamWriter sw = newF.AppendText();
                        sw.WriteLine(WLogTag);
                        sw.Close();
                        vFileToLog = newF;
                    }
                    else
                        vFileToLog = fi;
                }

                //cration d'un fichier s'il n'existe pas
                else
                {
                    string FilePath = CommandPath + @"\Log\Log.wlg";
                    FileInfo newF = new FileInfo(FilePath);
                    newF.Create().Close();
                    StreamWriter sw = newF.AppendText();
                    sw.WriteLine(WLogTag);
                    sw.WriteLine(DateTime.Now.ToString() + CellSeparator
                        + LogType.Information + CellSeparator + "First Log" + CellSeparator + "First Log" + CellSeparator);
                    sw.Close();
                    vFileToLog = newF;
                }
            }
            catch (Exception)
            {
                throw;
            }
        }
        /// <summary>
        /// Create an instance of log with the given path
        /// </summary>
        /// <param name="_Path">The path of the log file</param>
        /// <returns>Instance if the log file at this path</returns>
        /// <exception cref="FileNotFoundException">If the file is not founf</exception>
        public static Log InstanceOf(string _Path)
        {
            Log l = new Log();
            l.vFileToLog = new FileInfo(_Path);

            if(File.Exists(_Path))
                return l;
            else
                throw new FileNotFoundException();
        }
        #endregion

        #region Debug
        /// <summary>
        /// Gets the command line of this running application
        /// </summary>
        private string CommandLine { get { return Environment.CommandLine; } }
        /// <summary>
        /// Gets the execution path of this running application
        /// </summary>
        private string CommandPath
        {
            get
            {
                string Path = CommandLine;
                Path = Path.Remove(Path.LastIndexOf("\\"));
                Path = Path.Trim('\"');
                return Path;
            }
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Gets the cell separator
        /// </summary>
        public static string CellSeparator { get { return "|"; } }
        /// <summary>
        /// Gets the column separator
        /// </summary>
        public static string LineSeparator { get { return "\r\n"; } }
        /// <summary>
        /// Certification tag for a WLoGFile
        /// </summary>
        public static string WLogTag { get { return "<WLoG File>"; } }
        /// <summary>
        /// Gets the path of the file which contains the logs
        /// </summary>
        public string FileToLog { get { return vFileToLog.FullName; } }
        /// <summary>
        /// Gets the reference on the unique instance of Logs
        /// </summary>
        public static Log Instance { get { return GetInstance(); } }        
        /// <summary>
        /// Gets or sets the value for DebugMode
        /// </summary>
        /// <remarks>If TRUE and compilation on DEBUG it'll log otherwise not
        /// IF FALSE it'll log anyways</remarks>
        public bool DebugMode { get { return vDebugMode; } set { vDebugMode = value; } }
        #endregion
    }

    /// <summary>
    /// Donne le type d'entre de log
    /// </summary>
    /// <remarks>If a new line is added in the LogType, the log reader will understand
    /// automaticaly and integrate this new line in the program</remarks>
    public enum LogType
    {
        /// <summary>
        /// Miscellaneous
        /// </summary>
        Miscellaneous,
        /// <summary>
        /// Debug
        /// </summary>
        Debug,
        /// <summary>
        /// Error
        /// </summary>
        Error,
        /// <summary>
        /// Information
        /// </summary>
        Information,
        /// <summary>
        /// Information
        /// </summary>
        Report,
        /// <summary>
        /// Warning
        /// </summary>
        Warning,
        /// <summary>
        /// Flag to mark the logfile
        /// </summary>
        Flag,
    }
}
